-----------------------
-- PRINT.E  v1.2     --
-- by Gabriel Boehme --
-- updated 3/28/2000 --
-----------------------
--
-- This software is being contributed to the Public Domain.
-- Feel free to use it, abuse it, and/or modify it for any purpose.
-- Send bug reports or feature requests to:
--      gabrielboehme@hotmail.com
--
-- Flag for real time text readability and Indenting control
-- by Antonio Alessi 3/25/2006
-- as PRETTY_PUTS.E  v3.3
--

without warning
type pstring(sequence s)
-- verify that we have a valid Euphoria-representable string
-- (ASCII chars 32-255, 9, 10, 13)
object x

    for i = 1 to length(s) do
        x = s[i]
        if integer(x) then
            if x < ' ' then
                if not find(x, "\t\n") then             -- \r goes with \n only = {13,10} 
                    return 0
                end if
            elsif x > 255 then
                return 0
            end if
        else
        return 0
        end if
    end for
    return 1

end type

constant ESCAPE_STRING = {"\\n", "\\t", "\\\"", "\\\\", "\\r"},
         ESCAPED_CHARS = { '\n',  '\t',    '"',   '\\',  '\r'}

integer fn, Human
global integer maxLevels, indent_factor                 -- for the RTF indenting process
maxLevels = 0
indent_factor = 4

global function indentation( integer lev)
--sequence index
--index = sprintf( "%02d", lev)

    return repeat( ' ', lev *indent_factor)             -- ** SET THE SPACES or TABS YOU LIKE (*2,*3,*4..) for the 1st indent

end function
--- ========
procedure PrintEuString( sequence seq, integer level)
integer c, f

    puts( fn, '"')

    for i = 1 to length( seq) do
        c = seq[i]
        f = find( c, ESCAPED_CHARS)

        if f and not Human then                         --. **
            puts( fn, ESCAPE_STRING[f])
        else
            puts( fn, c)
            if c = 10 and level >0 then
                puts( fn, indentation( level))          -- indents every row in no-wrap mode
            end if
        end if
    end for

    puts(fn, '"')

end procedure
--- =========
procedure Print( sequence s, integer level)             -- **  level of indentation
integer len
object x

    len = length( s)
    if len and pstring( s) then                          -- print the sequence out as a string
        PrintEuString( s, level)
    else                                                -- print the sequence out as...well, a sequence
        if Human then
            level+= 1                                   -- ** indentation factor
            if level then                               -- not for the 1st line "{"
                puts(fn, '\n'&indentation( level))
            end if
            if level >maxLevels -1 then                 -- maxLevels must exist,
                maxLevels = level +1                    -- while level must start at 0
            end if
        end if
        puts(fn, '{')
        for i = 1 to len do
            x = s[i]
            if atom(x) then                             -- print out the numeric value
            	if x = 0 then
            		x = ZEROsym                         -- printf( 1, "%s", 0) prints NOTHING (???)
            	end if
                printf( fn, iff( Human, "%s", "%g"), x) -- adjust the non alpha bytes as you like
            else
                Print( x, level)
            end if
            if i< len then
                puts( fn, ',')
            end if
        end for
        puts(fn, '}') 
        
        puts( fn, 10)
        if level >1 then
        	puts( fn, indentation( level -1))	
        end if                
    end if

end procedure
--- =========
-- print out any Euphoria object, with strings in quotes
global procedure pretty_puts( integer file, object data, object read)

   if atom( data) then                                  -- just an atom, printf the value
      printf( file, "%g", data)
   else                                                 -- a sequence!
   	
	    if not find( 255, data) then                    -- replaces zero value in most cases..
	    	ZEROsym = 255
	    else
	    	ZEROsym = 0                                 -- no changes, hope in the future    
	    end if    
	    	
      maxLevels = 0
      if atom( read) then                               -- pretty_print(1, "",{8}) compatible
	      Human = read                                  -- 0 = allow / 1 = exclude ESCAPED_CHARS
      else	
      	 Human = read[1]
      end if
      fn = file
--trace(1)
      Print( data, -1)                                  -- ** -1 = not indenting the 1st line
   end if

end procedure
